/*
 * novand.c
 *
 * Nova buffer manager network daemon program
 *
 *              version 1.03  08-OCT-1992 Masaharu NOMACHI   (KEK online)
 *              version 1.08  18-JAN-1994 Yasuo TAKEUCHI     (T.I.T.)
 */

#include "novanet.h"

char unknown[] = {"unknown"};

void child_end();

main(argc,argv) int argc; char **argv; {
  int sock, length;
  struct sockaddr_in server, cliant;
  struct hostent *hp;
  int msgsock, rval;
  char *name;
  int status;
  int pid;

  /* Create socket */
  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock < 0) {
    printf("** ERROR ** opening stream socket. errno=%X\n",errno);
    exit(1);
  }

  /* Name socket using wildcards */
  server.sin_family = AF_INET;
  server.sin_addr.s_addr = INADDR_ANY;
  server.sin_port = htons(NOVA_PORT);
  if (bind(sock, &server, sizeof(server)) == -1) {
    printf("** ERROR ** binding stream socket. errno=%X",errno);
    exit(1);
  }

  /* Find out assigned port number and print it out */
  length = sizeof(server);
  if (getsockname(sock, &server, &length) == -1) {
    printf("** ERROR ** getting socket name. errno=%X\n",errno);
    exit(1);
  }
  printf("Socket %d has port #%d\n",sock,ntohs(server.sin_port));

  /* add signal handler */
  signal(SIGCHLD, child_end);

  /* Start accepting connections */
  status = listen(sock,5);
  if (status == -1) {
    printf(" ***** listen ****** errno = %X\n",errno);
    exit(1);
  }

  do {
      length = sizeof(cliant);
      msgsock = accept(sock,&cliant,&length);
      if (msgsock >= 0) {
	  hp = gethostbyaddr(&cliant.sin_addr,sizeof(struct in_addr),AF_INET);
	  if (hp == 0) 
	      name = unknown;
	  else
	      name = hp->h_name;
	  printf(" socket #%d   message from %s.\n",msgsock,name);
	  rval = exec_novand(msgsock,argv[0]);
	  close(msgsock);
      }
      else {
	  if (msgsock == -1 && errno != EINTR) {
	      printf("** ERROR ** accept. errno=%X\n",errno);
	      exit(1);
	  }
      }
  } while(rval==0);

  exit(0);
}

exec_novand(msgsock,proc_name) int msgsock; char *proc_name; {
  char buf[1024];
  int rval, pid, pri, req;
  char *name, *index();
  int i;

  bzero(buf,sizeof(buf));
  /* System V version 
  memset(buf, '\0', sizeof(buf));
  */
  rval = read(msgsock,buf,1024);
  if (rval == 0) {
    printf("Ending connection\n");
    return 0;
  }
  if (rval < 0) {
    printf("** ERROR ** reading stream message. errno=%X\n",errno);
    return -1;
  }
  if (rval>100) {
    printf("** ERROR ** too long stream message.\n");
    return -2;
  }

  name = index(buf,':');
  if ((int)name == 0) return -2;

  name++;
  for(i=0;i<80;i++) {
    if (name[i]==':') break;
    if (name[i]==(char)0) break;
  }
  name[i] = (char)0;

  rval = sscanf(&name[i+1],"%d:%d",&pri,&req);
  if (rval !=2) return -2;

  printf(" name = %s pri=%d req=%d\n",name,pri,req);

  if (strncmp(buf,"tx",strlen("tx"))==0) {
    pid = fork();
    if (pid==0) {
      strcpy(proc_name,"novatx");
      exec_tx(msgsock,name,pri,req);
      close(msgsock);
      exit(0);
    }
  }
  else if (strncmp(buf,"rx",strlen("rx"))==0) {
    pid = fork();
    if (pid==0) {
      strcpy(proc_name,"novarx");
      exec_rx(msgsock,name,pri,req);
      close(msgsock);
      exit(0);
    }
  }
  else {
    printf("** ERROR ** Invalid message command %s\n",buf);
    return -2;
  }
  return 0;
}


void child_end() {
    while (wait3(0,WNOHANG,0) > 0)
        ;
    return;
}
