/*
 * Buffer manager execute functions
 *
 *	                16-JUL-1992	KEK	Masaharu NOMACHI
 *	version 1.02    30-SEP-1992	KEK	Masaharu NOMACHI
 *              version 1.10  14-Jul-1994 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.11   7-Jul-1995 Yasuo TAKEUCHI     (T.I.T.)
 */
#include "bm.h"
#include "bm_exec.h"
#include "bm_config.h"

/*** variable length ***/
extern int max_buffer_size;
/* proto types */
void swap_buffer(), devide_buffer();
int get_null_buffer();
struct EXEC_buffer  *get_free_buffer(), *get_buffer(), *get_wait_buffer();

struct EXEC_process *consumer;
struct EXEC_process *wait_producer, *busy_producer;
struct EXEC_buffer  *free_buffer;
int  threshold;

int exec_init(thr) int thr; {

  threshold = thr;

  /* Initialize the link top */
  free_buffer   = NULL_buffer;
  wait_producer = NULL_process;
  busy_producer = NULL_process;
  consumer      = NULL_process;
  
  return 0;
}

int exec_buff_install(table,offset,size)
     struct BMT_buffer *table; int offset, size;
{
  struct EXEC_buffer *buffer;
  
  /* Create buffer data base */
  buffer = (struct EXEC_buffer *)malloc(sizeof(struct EXEC_buffer));
  if ((int)buffer==-1) return -1;

  /* Initialize the table and data base */
  table->b_exec         = (char *)buffer;
  buffer->table         = table;
  buffer->type          = 0;
  buffer->significance  = 0;
  buffer->status        = BS_null;
  buffer->offset        = offset;
  buffer->max_size      = size;
  put_buffer(&free_buffer,buffer);

  return 0;
}

int exec_process_install(p_table,type,priority,n_request)
    struct BMT_process *p_table; int type, priority, n_request; {

  struct EXEC_process *p;
  struct EXEC_buffer  *b;
  
  /* Create process data base */
  p = (struct EXEC_process *)malloc(sizeof(struct EXEC_process));
  if ((int)p == -1) return -1;
  
  /* Install it on the pointer table */
  p_table->p_exec = (char *)p;
  
  /* initialize the parameters */
  p->table      = p_table;
  p->type       = type;
  p->wait       = NULL_buffer;
  p->active     = NULL_buffer;
  p->n_request  = n_request;
  p->n_active   = 0;
  if (priority > 0) {
    p->priority   = priority;
    p->status     = PS_consumer;
    if (put_process(&consumer,p)) return -1;
  }
  else {
    p->priority   = -priority;
    p->status     = PS_wait;
    if (put_process(&wait_producer,p)) return -1;
    stimulate_producer();
  }
  return 0;
}

int exec_remove(p_table) struct BMT_process *p_table; {
  struct EXEC_process *p, *q;

  /* get the process removed */
  p = (struct EXEC_process *)(p_table->p_exec);
  
  /* remove it from the link */
  if      (p->status == PS_consumer) remove_process(&consumer,p);
  else if (p->status == PS_busy)     remove_process(&busy_producer,p);
  else if (p->status == PS_wait)     remove_process(&wait_producer,p);
  p_table->p_exec = (char *)0;


  if ((p->status == PS_consumer) && (p->next != p)){
    q = p->next;
    append_buffer(&(q->wait),p->active);
    append_buffer(&(q->wait),p->wait);
    stimulate_consumer(q);
  }
  else {
      /*** variable length ***/
      if (max_buffer_size > 0) {
	  append_free_buffer(p->active);
	  append_free_buffer(p->wait);
      }
      else {
	  append_buffer(&free_buffer,p->active);
	  append_buffer(&free_buffer,p->wait);
      }
      stimulate_producer();
  }
  /* Back it to free */
  free(p);

  return 0;
}

int bm_pause(p_table,control) struct BMT_process *p_table; char *control; {
  struct EXEC_process *p, *q;

  /* get the process */
  p = (struct EXEC_process *)(p_table->p_exec);
  
  if (p->status == PS_wait) {
    p->status = PS_busy;
    remove_process(&wait_producer,p);
    put_process(&busy_producer,p);
  }

  p->n_request = 0;
  if (control[0]=='d') {
    p->n_active = 0;
    if ((p->status == PS_consumer) && (p->next != p)){
      q = p->next;
      append_buffer(&(q->wait),p->active);
      append_buffer(&(q->wait),p->wait);
      p->active = NULL_buffer;
      p->wait   = NULL_buffer;
      stimulate_consumer(q);
    }
    else {
	/*** variable length ***/
	if (max_buffer_size > 0) {
	    append_free_buffer(p->active);
	    append_free_buffer(p->wait);
	}
	else {
	    append_buffer(&free_buffer,p->active);
	    append_buffer(&free_buffer,p->wait);
	}
	p->active = NULL_buffer;
	p->wait   = NULL_buffer;
	stimulate_producer();
    }
  }
  return 0;
}

int bm_resume(p_table,request) struct BMT_process *p_table; int request; {
  struct EXEC_process *p, *q;

  /* get the process */
  p = (struct EXEC_process *)(p_table->p_exec);
  
  p->n_request = request;
  /*** variable length ***/
  if (max_buffer_size > 0) {
      append_free_buffer(p->active);
      append_free_buffer(p->wait);
  }
  else {
      append_buffer(&free_buffer,p->active);
      append_buffer(&free_buffer,p->wait);
  }
  p->n_active  = 0;
  p->active = NULL_buffer;
  p->wait = NULL_buffer;

  if (p->status == PS_busy) {
    p->status = PS_wait;
    remove_process(&busy_producer,p);
    put_process(&wait_producer,p);
  }
  stimulate_producer();
  return 0;
}

int bm_request(p_table) struct BMT_process *p_table; {
  struct EXEC_process *p, *q;

  /* get the process */
  p = (struct EXEC_process *)(p_table->p_exec);
  
  if (p->n_request <= 0) p->n_request -= 1;
  return 0;
}

int bm_done(p_table,b_table,type,size,significance)
     struct BMT_process *p_table; struct BMT_buffer *b_table;
     int type, size, significance; 
{
  register int i;
  struct EXEC_process *p, *q, *c;
  struct EXEC_buffer  *b, *buff, *buf1;
  
  /* get the processes and the buffer */
  p = (struct EXEC_process *)(p_table->p_exec);
  b = (struct EXEC_buffer  *)(b_table->b_exec);

  /* search done buffer */
  for (i = 0; i < p->n_active; i++) {  
      if (p->active == b)
	break;
      p->active = p->active->next;
  }
  if (p->active != b) {
      printf("Invalid address\n");
      return BME_INVADR;
  }
  --(p->n_active);

  if (p->status == PS_consumer) {
    if (significance != 0) b->significance = significance;
    if (size > 0) {
      if (size > b->max_size) b->size = b->max_size;
      else                    b->size = size;
    }
    q = p->next;
    if ((q == consumer) || (p->n_request <= 0)) {
      forward_buffer(p,&free_buffer);
      stimulate_producer();
    }
    else {
      forward_buffer(p,&(q->wait));
      stimulate_consumer(q);
    }
    stimulate_consumer(p);
  }
  else {
    b->significance = significance;
    if (size > b->max_size) b->size = b->max_size;
    else                    b->size = size;
    c = consumer;
    buff = get_buffer(&(p->active));
    if ((p->status == PS_busy) && (p->n_request > 0)) {
      p->status = PS_wait;
      remove_process(&busy_producer,p);
      put_process(&wait_producer,p);
    }

    /* Forward the buffer */
    if (c!=NULL_process) {
	/*** variable length ***/
	if (max_buffer_size > 0 && buff->size < buff->max_size) {
	    if (get_null_buffer(&buf1) == 0) {
		swap_buffer(buff, buf1);
		devide_buffer(buf1, buff, buf1->size);
		put_free_buffer(buf1);
	    }
	}
	put_buffer(&(c->wait),buff);
	stimulate_consumer(c);
    }
    else {
	/*** variable length ***/
	if (max_buffer_size > 0)
	  put_free_buffer(buff);
	else
	  put_buffer(&free_buffer,buff);
    }
    stimulate_producer();
  }
  return 0;
}

stimulate_consumer(p) struct EXEC_process *p; {
  struct EXEC_buffer  *buff;
  struct EXEC_process *q;

  if (p->wait == NULL_buffer) return;
  if (p->n_request == 0) {
    q = p->next;
    if (q != consumer) {
      append_buffer(&(q->wait),p->wait);
      p->wait = NULL_buffer;
      stimulate_consumer(q);
    }
    else {
	/*** variable length ***/
	if (max_buffer_size > 0) {
	    append_free_buffer(p->wait);
	}
	else {
	    append_buffer(&free_buffer,p->wait);
	}
	p->wait = NULL_buffer;
	stimulate_producer();
    }
    return;
  }
#ifdef BMDEBUG
  printf("STIMULATE    consumer %d (active:%d) (request:%d)\n",
	 p->table->id,p->n_active,p->n_request);
#endif
  while ((p->n_active < p->n_request) || (p->n_request < 0)) {
    buff = get_buffer(&(p->wait));
    put_buffer(&(p->active),buff);
    if (bm_req_test(p->table,buff->table)) {
      (p->n_active) += 1;
      if (p->n_request < 0) p->n_request += 1;
      bm_send(p->table,buff->table,buff->offset,buff->size);
      buff->status = BS_sent;
    }
    else {
      buff->status = BS_passed;
    }
    if (p->wait == NULL_buffer) break;
    if (p->wait->significance > p->priority) break;
  }

  /* Forward the passed buffer */
  if (p->active != NULL_buffer) {
    q = p->next;
    if (q==consumer) {                                 
      if (p->active->status != BS_sent) forward_buffer(p,&free_buffer);
      stimulate_producer();
    }
    else {
      if (p->active->status != BS_sent) forward_buffer(p,&(q->wait));
      stimulate_consumer(q);
    }
  }
  return;
}
    
stimulate_producer() {
    struct EXEC_process *p, *c, *last;
    struct EXEC_buffer  *b;

    if (wait_producer == NULL_process) return;
    last = wait_producer->previous;
    while(wait_producer != NULL_process) {
	/* get a free buffer */
	/*** variable length ***/
	if (max_buffer_size > 0)
	    b = get_free_buffer();
	else
	    b = get_buffer(&free_buffer);
	if (b == NULL_buffer) {
	    /* get from consumer processes */
	    c = consumer;
	    if (c == NULL_process) break;
	    do {
		c = c->previous;
		/*** variable length & fixed bug? ***/
		if (max_buffer_size > 0) {
		    b == NULL_buffer;
		    while (b == NULL_buffer && 
			   (b = get_wait_buffer(c)) != NULL_buffer) {
			put_free_buffer(b);
			b = get_free_buffer();

		    }
		    if (b != NULL_buffer) break;
		}
		else {
		    b = get_wait_buffer(c);
		    break;
		}
	    } while(c != consumer);
	    if (b == NULL_buffer) break;
	}
	p = wait_producer;
#ifdef BMDEBUG
	printf("STIMULATE producer    %d (active:%d)\n",
	       p->table->id,p->n_active);
#endif
	b->size = b->max_size;
	bm_send(p->table,b->table,b->offset,b->size);
	put_buffer(&(p->active),b);
	(p->n_active) += 1;
	if (p->n_active >= p->n_request) {
	    remove_process(&wait_producer,p);
	    put_process(&busy_producer,p);
	    p->status = PS_busy;
	}
	else wait_producer = p->next;
	if (last == p) break;
    }
    return;
}
    
forward_buffer(p,destination)
     struct EXEC_process *p; struct EXEC_buffer **destination; 
{
  struct EXEC_buffer  *buff;

  if (p->active == NULL_buffer) return;
  while(1) {
    buff = get_buffer(&(p->active));
    /*** variable length ***/
    if (*destination == free_buffer && max_buffer_size > 0)
      put_free_buffer(buff);
    else
      put_buffer(destination,buff);
    if (p->active == NULL_buffer) break;
    if (p->active->status != BS_passed) break;
  }
  return;
}
