/*
 * mgr.c
 *
 *     Manager library for Super-Kamiokande online DAQ system
 *     ported from mgrlib.c
 *
 *              version 1.12  20-Jul-1995 Yasuo TAKEUCHI     (ICRR)
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <sys/utsname.h>

/***** definitions ***************************************/
#define MGR_MSGLEN       80             /* message buffer size */
#define MGR_PORT         "5201"         /* node port */


/***** functions *****************************************/
void nodename( char *name )
{
    struct utsname name_t;
 
/*-- set nodename --*/
    uname(&name_t);
    strcpy( name, (const char *)name_t.nodename);
}


/***** main routine **************************************/
int mgr_open(node, pname)
char *node;
char *pname;
{
    struct servent *sp;
    struct hostent *hp;
    struct sockaddr_in sin;
    int sd, i, len;
    char buf[MGR_MSGLEN],  hostname[10];

    /* check host name on table */
    if((hp = gethostbyname(node)) == NULL){
        printf("mgr_open: %s is unknown host name\n",node);
        return -1;
    }

    /* get local node name */
    nodename(hostname);

    /* create socket */
    if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        perror("mgr_open: socket");
        return -1;
    }

    /* copy network address */
    memcpy( (char *) &sin.sin_addr, (char *) hp->h_addr, hp->h_length);
    sin.sin_family = AF_INET; 
    sin.sin_port = htons(atoi(MGR_PORT)); 

    /* connection request */
    if(connect(sd, (struct sockaddr *) &sin, sizeof sin) < 0){
        perror("mgr_open: connect");
        return -1;
    }
    
    /* send host name and process name */
    sprintf(buf, "%s %s", hostname, pname);
    if((len = write(sd, buf, MGR_MSGLEN)) < 0) return -1;
    /* normal return */
    return sd;
}
    
int mgr_close(sd)
int sd;
{
    close(sd);
    return 0;
}
