/*
 * novad.c
 *
 * Nova buffer manager daemon program
 *
 *              version 0.1   24-APR-1992 kekonline M. Nomachi
 *              version 0.3   22-JUN-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.02  01-OCT-1992 Masaharu NOMACHI   (KEK online)
 *              version 1.06  12-APR-1993 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.07  27-NOV-1993 Yasuo TAKEUCHI     (T.I.T.)
 */

#include "novad.h"

int pid_buffer_manager;

main(argc, argv) int argc; char **argv; {

  struct nova_def novad;
  struct sockaddr_un server;
  int i, sd, msd, rval;
  int pid, key, novaid;

  int msgid, shmid;
  int child_end();

  /*
   * Rise process priority if possible
   */
  nice(NV_NICE_VALUE);

  /*
   * Lock process
   */
/*  plock(PROCLOCK); */

  /*
   * Check arguments
   */
  if (exam_arg(argc, argv, &novad) < 0) exit(1);

  /*
   * Create socket
   */
  if ((sd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
    perror("novad: opening stream socket");
    exit(1);
  }

  /*
   * name socket
   */
  server.sun_family = AF_UNIX;
  strcpy(server.sun_path, novad.name);

  /* delete previous socket, if exist */
  /* unlink(novad.name); */

  if (bind(sd, &server, sizeof(struct sockaddr_un))) {
    perror("novad: binding stream socket");
    exit(1);
  }

  /* change socket permittion (for Ultrix) */
/*  chmod(novad.name, 0777); */

  /*
   * create "message" and "shared memory"
   */
  if (init_daemon_msg(&novad) < 0) {
    unlink(novad.name);
    exit(1);
  }
  if (init_bufmgr_msg(&novad) < 0) {
    unlink(novad.name);
    exit(1);
  }
  if ((shmid = shmget(IPC_PRIVATE, novad.total_size, IPC_CREAT|0666)) < 0) {
    msgctl(msgid, IPC_RMID, NULL);
    perror("novad: shmget");
    unlink(novad.name);
    exit(1);
  }

  /* lock shared memory */
  shmctl(shmid, SHM_LOCK, NULL);

  signal(SIGCHLD, child_end);
  if ((i=fork()) != 0) {
    printf("novad's pid = %d\n",i);
    exit(0);
  }
  /*
   * start buffer manager 
   */
  if ((pid_buffer_manager = fork()) == 0) {
    /* child process (buffer_manager) */
    close(sd);
    strcpy(argv[0], BUFM_NAME);
    buffer_manager(&novad);
    exit(0);
  }
  else if (pid_buffer_manager == -1)
    perror("novad: fork buffer manager");

  /*
   * start accept
   */
  listen(sd, novad.n_process);
  while(pid_buffer_manager > 0) {
    if ((msd = accept(sd, 0, 0)) != -1) {
      if (pid_buffer_manager > 0) {
	if(fork() == 0) {
	  /* child process (server) */
	  close(sd);
	  strcpy(argv[0], SERV_NAME);
	  if (exec_install(msd, &novad, shmid, &pid, &novaid) == 0) {
	    nv_server(msd, &novad, pid, novaid);
	  }
	  exit(0);
	}
      }
      close(msd);
    }
  }
#ifdef DEBUG
  printf("novad: Buffer manager has been killed\n");
#endif
  shmctl(shmid, IPC_RMID, NULL);
  msgctl(novad.bufmgr_msgid, IPC_RMID, NULL);
  msgctl(novad.daemon_msgid, IPC_RMID, NULL);
  unlink(novad.name);
  close(sd);
}

int child_end() {
  int pid;

  while ((pid = wait3(0,WNOHANG,0)) > 0) {
      if (pid == pid_buffer_manager) pid_buffer_manager = 0;
  }
/*
  pid = wait(0);
  if (pid == pid_buffer_manager) pid_buffer_manager = 0;
*/
  return 0;
}
