/*
 * novad_exam_arg.c
 *
 *    Examine the arguments for novad
 *                                    KEK online group
 *                                    Masaharu NOMACHI
 *                                    Tokyo Institute of Technology
 *                                    Yasuo TAKEUCHI
 *
 *              version 0.2   08-JUN-1992 Masaharu NOMACHI
 *              version 0.33  28-AUG-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.01  26-SEP-1992 Masaharu NOMACHI
 *              version 1.05  24-NOV-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.06  12-APR-1993 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.07  27-NOV-1993 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.10  14-Jul-1994 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.10sk  15-May-1995  Yasuo Takeuchi     (ICRR)
 *              version 1.12  20-Jul-1995 Yasuo TAKEUCHI     (ICRR)
 *
 *  novad [options] name
 *       -b #     buffer size in K-byte
 *       -B #     buffer size in byte
 *       -n #     number of buffers
 *       -p #     max process number
 *       -v #     max variable free buffer size at producer in K-byte
 *       -V #     max variable free buffer size at producer in byte
 *       -h hostname    hostname for sending messages (for SK)
 */

#include "novad.h"

/*** variable length ***/
extern int max_buffer_size;
extern int max_buffer_size_org;

show_usage()
{
    printf("\n");
    printf("  NOVA Buffer Manager version 1.12sk 20-Jul-1995");
    printf("written by M.Nomachi, Y.Takeuchi\n");
    printf("  usage: novad [options] name [options] \n");
    printf("\n");
    printf("       -b #    buffer size in K-byte             (default 16)\n");
    printf("       -B #    buffer size in byte                           \n");
    printf("       -n #    number of buffers                 (default  8)\n");
    printf("       -p #    max number of client processes    (default 20)\n");
    printf("       -v #    max free buffer length in K-byte  (default  0)\n"); 
    printf("       -V #    max free buffer length in byte                \n"); 
    printf("                 <= 0: fixed length buffer                \n");
    printf("                  > 0: variable length buffer             \n");
    printf("       -h hostname    host to send messages      (default not send)\n"); 
    printf("\n");

    return 0;
}

int exam_arg(argc,argv,p) int argc; char *argv[]; struct nova_def *p; {
    int i, sd;
    char node[100];  /* hostname to send messages */
    node[0] = '\0';

    /* set default values */
    p->buff_size      = (16<<10);  /* buffer size = 16K byte          */
    p->n_buffer       =  8;        /* default 8 buffer.               */
    p->n_process      = 20;        /* default max 20 process          */
    max_buffer_size   =  0;        /* fixed length buffer             */
    p->name[0] = '\0';

    for (i = 1; i < argc; i++) {
	if (argv[i][0] == '-') {
	    if (argv[i][1] == 'b') {
		i++; p->buff_size = atoi(argv[i]) << 10;
	    }
	    else if (argv[i][1] == 'B') {
		i++; p->buff_size = atoi(argv[i]);
	    }
	    else if (argv[i][1] == 'n') {
		i++; p->n_buffer = atoi(argv[i]);
	    }
	    else if (argv[i][1] == 'p') {
		i++; p->n_process = atoi(argv[i]);
	    }
	    else if (argv[i][1] == 'v') {
		i++; max_buffer_size = atoi(argv[i]) << 10;
	    }
	    else if (argv[i][1] == 'V') {
		i++; max_buffer_size = atoi(argv[i]);
	    }
	    else if (argv[i][1] == 'h') {
		i++; strcpy(node, argv[i]);
	    }
	}
	else if (strlen(argv[i]) <= NV_MAXNAME)
	    strcpy(p->name, argv[i]);
    }

    if (p->name[0] == '\0') {
	show_usage();
	exit(1);
    }

    p->total_size = p->buff_size * p->n_buffer;

    if ((p->total_size > NV_MAXTOTAL) || (p->total_size < 1)) {
	printf("Invalid shared memory size %d\n", p->total_size);
	return -1;
    }
    if (p->buff_size < 1) {
	printf("Invalid size of buffer %d\n", p->buff_size);
	return -1;
    }
    if (p->n_buffer < 1) {
	printf("Invalid number of buffer %d\n", p->n_buffer);
	return -1;
    }
    if (p->n_process < 1) {
	printf("Invalid number of max process %d\n", p->n_process);
	return -1;
    }

    max_buffer_size_org = max_buffer_size;

    /* check and send messages */
    if (node[0] != '\0') {
	if (mgr_open(node, "novad") < 0) {
	    perror("mgr_open failed!");
	}
	else {
	    printf("mgr_open succeeded! (node = %s)\n", node);
	}
    }

    return 0;
}
